##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Basic payment tool.
$Id: MemberDataTool.py,v 1.5 2001/06/04 03:30:05 tseaver Exp $
"""
__version__='$Revision: 1.5 $'[11:-2]


from Products.CMFCore.utils import UniqueObject, getToolByName, _getAuthenticatedUser
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Globals import package_home, Acquisition, Persistent, DTMLFile
import Globals, os
from AccessControl.Role import RoleManager
from BTree import BTree
from ZPublisher.Converters import type_converters
from Acquisition import aq_inner, aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import ViewManagementScreens, setDefaultRoles
from Products.CMFCore import CMFCorePermissions

_marker = []  # Create a new marker object.

_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

ManagePayments = 'Register and clear payments'
setDefaultRoles(ManagePayments, ('Manager'))

ManageTransactions = 'Clear and register usage'
setDefaultRoles(ManageTransactions, ('Manager'))

class PaymentTool (UniqueObject, SimpleItem, PropertyManager):
    '''This tool wraps user objects, making them act as Member objects.
    '''
    id = 'portal_payment'
    meta_type = 'Simple Payment Tool'
    _v_temps = None
    _properties = ()

    security = ClassSecurityInfo()

    manage_options=( ( { 'label' : 'Overview'
                       , 'action' : 'manage_overview'
                       }
                     , { 'label' : 'Pending Transactions'
                       , 'action' : 'manage_pending'
                       }
		     , { 'label' : 'Dept'
		       , 'action' : 'manage_showDept'
		       }
                     )
                   + PropertyManager.manage_options
                   + SimpleItem.manage_options
                   )

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainPaymentTool', _dtmldir )

    security.declareProtected( ManagePayments
                             , 'manage_pending')
    manage_pending = DTMLFile('paymentsPending', _dtmldir )

    security.declareProtected( ManagePayments
                             , 'manage_showDept')

    manage_showDept = DTMLFile('paymentDept', _dtmldir )

    security.declareProtected( CMFCorePermissions.ViewManagementScreens
                             , 'getContentsInformation',)


    def __init__(self):
        self._members = BTree()
	self._supliers = BTree()


    def setGUID(self, guid):
	user = self.wrapCustomer(_getAuthenticatedUser(self))
	user.setGUID()

    def getUser(self, guid):
	for user in self._members.values():
            if user.getGUID() == guid:
	       return user.getId()
        return "Guest"

    #
    # Payment registering and clearing
    #
    security.declarePublic('getCredits')
    def getCredits(self):
	return self.getUserCredits(_getAuthenticatedUser(self))


    security.declareProtected(ManagePayments, 'getUserCredits')
    def getUserCredits(self, user):
	user = self.wrapCustomer(user)
	return user.Credits()
	
    security.declareProtected(ManagePayments, 'registerUserPayments')
    def registerUserPayment(self, user, credit):
        '''
	  Registers a payment for in the system
	'''
	credit = int(credit)
	user = self.wrapCustomer(user)
	user.registerPayment(credit)
	
    security.declareProtected(ManagePayments, 'clearUserPayments')
    def clearUserPayments(self, user):
	user = self.wrapCustomer(user)
	user.clearPayments()

    security.declarePrivate('payment')
    def payment(self, resource, amount):
	cur = 0
	if self._supliers.has_key(resource.Creator()):
          cur = self._supliers[resource.Creator()]
	self._supliers[resource.Creator()] = cur + amount
	
    security.declarePublic('getDept')
    def getDept(self):
	user = _getAuthenticatedUser(self).getId()
	if self._supliers.has_key(user):
	  return self._supliers[user]
        return 0

    security.declareProtected(ManagePayments, 'clearDept')
    def clearDept(self, user):
	self._supliers[user] = 0

    security.declareProtected(ManagePayments, 'listDept')
    def listDept(self):
	depts = []
	for user in self._supliers.keys():
	  depts.append({'User': user, 'Dept': self._supliers[user]})

	return tuple(depts)

    #
    # Transaction clearing and registering
    #
    security.declareProtected(ManageTransactions, 'clearTransaction')
    def clearTransaction(self, user, expected=0):
        user = self.wrapCustomer(user)
	if user.Credits() > expected:
	    return true
        else:
	    return false
    
    security.declareProtected(ManageTransactions, 'registerTransaction')
    def registerTransaction(self, user, resource, amount):
        user = self.wrapCustomer(user)
	user.registerTransaction(resource, amount)

	
    #
    #   'portal_memberdata' interface methods
    #
#    security.declarePrivate('getMemberDataContents')
#    def getMemberDataContents(self):
#        '''
#        Return the number of members stored in the _members
#        BTree and some other useful info
#        '''
#        membertool   = getToolByName(self, 'portal_membership')
#        members      = self._members
#        user_list    = membertool.listMemberIds()
#        member_list  = members.keys()
#        member_count = len(members)
#        orphan_count = 0
#
#        for member in member_list:
#            if member not in user_list:
#                orphan_count = orphan_count + 1
#
#        return [{ 'member_count' : member_count,
#                  'orphan_count' : orphan_count }]
#
#    security.declarePrivate('pruneMemberDataContents')
#    def pruneMemberDataContents(self):
#        '''
#        Compare the user IDs stored in the member data
#        tool with the list in the actual underlying acl_users
#        and delete anything not in acl_users
#        '''
#        membertool= getToolByName(self, 'portal_membership')
#        members   = self._members
#        user_list = membertool.listMemberIds()
#
#        for tuple in members.items():
#            member_name = tuple[0]
#            member_obj  = tuple[1]
#            if member_name not in user_list:
#                del members[member_name]

    security.declarePrivate('wrapUser')
    def wrapCustomer(self, u):
        '''
        If possible, returns the Member object that corresponds
        to the given User object.
        '''
        id = u.getUserName()
        members = self._members
        if not members.has_key(id):
            # Get a temporary member that might be
            # registered later via registerMemberData().
            temps = self._v_temps
            if temps is not None and temps.has_key(id):
                m = temps[id]
            else:
                base = aq_base(self)
                m = CreditData(base, id)
                if temps is None:
                    self._v_temps = {id:m}
                else:
                    temps[id] = m
        else:
            m = members[id]
        # Return a wrapper with self as containment and
        # the user as context.
        return m.__of__(self).__of__(u)

    security.declarePrivate('registerMemberData')
    def registerMemberData(self, m, id):
        '''
        Adds the given member data to the _members dict.
        This is done as late as possible to avoid side effect
        transactions and to reduce the necessary number of
        entries.
        '''
        self._members[id] = m

Globals.InitializeClass(PaymentTool)


class CreditData (SimpleItem):
    security = ClassSecurityInfo()

    def __init__(self, tool, id):
        self.id = id
	self.money = 0
	self.guid = "000000-0000-0000-0000-0000"
        # Make a temporary reference to the tool.
        # The reference will be removed by notifyModified().
        self._tool = tool

    security.declarePrivate('notifyModified')
    def notifyModified(self):
        # Links self to parent for full persistence.
        tool = getattr(self, '_tool', None)
        if tool is not None:
            del self._tool
            tool.registerMemberData(self, self.getId())

    security.declarePrivate('registerTransaction')
    def registerTransaction(self, resource, amount):
	tool = self.getTool()
        tool.payment(resource, amount) 	
	self.money = self.money - amount
	self.notifyModified()

    security.declarePrivate('setGUID')
    def setGUID(self, guid):
	self.guid = guid

    security.declarePrivate('getGUID')
    def getGUID(self, guid):
	return self.guid
        	
    security.declarePrivate('registerPayment')
    def registerPayment(self, credits):
	self.money = self.money + credits
	self.notifyModified()

    security.declarePrivate('clearPayments')
    def clearPayments(self):
	self.notifyModified()
	return None

    security.declarePrivate('Credits')
    def Credits(self):
	return self.money
	    

    security.declarePublic('getUser')
    def getUser(self):
        # The user object is our context, but it's possible for
        # restricted code to strip context while retaining
        # containment.  Therefore we need a simple security check.
        parent = aq_parent(self)
        bcontext = aq_base(parent)
        bcontainer = aq_base(aq_parent(aq_inner(self)))
        if bcontext is bcontainer or not hasattr(bcontext, 'getUserName'):
            raise 'CreditDataError', "Can't find user data"
        # Return the user object, which is our context.
        return parent

    def getTool(self):
        return aq_parent(aq_inner(self))

    def __str__(self):
        return self.getUserName()

    ### User object interface ###

    security.declarePublic('getUserName')
    def getUserName(self):
        """Return the username of a user"""
        return self.getUser().getUserName()

    security.declarePublic('getId')
    def getId(self):
        """Get the ID of the user. The ID can be used, at least from
        Python, to get the user from the user's
        UserDatabase"""
        return self.getUser().getId()


Globals.InitializeClass(CreditData)
